import { ComponentType, ComponentPropsWithRef, PropsWithRef } from 'react';
import { CatalogComponents } from '../types';
interface IProps<T> {
    /**
     * Path to the required component in the catalog. component can take any
     * value supported by `catalog.getComponent`.
     *
     * ### Note
     * The `CatalogComponents` interface matches last, because until its
     * augmented it matches everything.
     * @see https://stackoverflow.com/a/58512513/1238150
     */
    component: T extends Record<string, unknown> ? keyof T : T extends (infer R)[] ? R : T extends string ? string : T extends CatalogComponents ? keyof CatalogComponents : any;
    /**
     * when no component is found, a fallbackComponent is rendered instead
     */
    fallbackComponent?: ComponentType<ComponentPropsWithRef<any>> | string;
    /**
     * any other component property is just taken as is
     */
    [property: string]: any;
}
/**
 * `CatalogComponent` is wrapped `withCatalog` by default and is capable of
 *
 * - rendering requested component from the catalog
 * - passing props to the requested component
 * - render a fallback component when `props.fallback` (node) is defined
 * - returns null (does not throw an error) when component does not exist in
 *   the catalog
 *
 * Example:
 * ```jsx
 * <CatalogComponent component="Button" hello="world" fallback={() => <div>not found</div>} />
 * ```
 *
 * `CatalogComponent` will now render the `<Button />` component (if it is part
 * of the catalog and pass `hello` as a prop to the Button). If the component
 * would not exist it will render the fallback.
 */
declare const CatalogComponentWrapper: {
    <T extends unknown>({ ref, ...rest }: IProps<T>): JSX.Element;
    displayName: string;
};
/**
 * CatalogComponentWrapper is used to type React.forwardRef properly
 *
 * inspired by:
 * @see https://stackoverflow.com/a/58473012/1238150
 */
export default CatalogComponentWrapper;
//# sourceMappingURL=catalog-component.d.ts.map